﻿using System;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Windows.Data;

namespace DataFormTest 
{
    [CustomValidation(typeof(EmployeeValidator), "ValidateEmployee")]
    public class Employee : ModelBase, IEditableObject
    {
        private const string phoneExpression = @"^\(\d\d\d\) \d\d\d\-\d\d\d\d$";
        private const string emailExpression = @"^\w+([-+.]\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*$";

        private Employee cache { get; set; }
        //[Display(AutoGenerateField = false)]

        private int employeeId;
        [Display(Name="Id", Description = "Employee Id")]
        [Key]
        [Editable(false)]
        public int EmployeeId
        {
            get { return employeeId; }
            set
            {
                if (employeeId == value) return;
                employeeId = value;
                FirePropertyChanged("EmployeeId");
            }
        }

        private string firstName;
        [Required(ErrorMessage = "Required field")]
        [StringLength(40, ErrorMessage = "Cannot exceed 40")]
        [Display(Name = "First Name", Description = "Employee's first name")]
        public string FirstName
        {
            get { return firstName; }
            set
            {
                if (firstName == value) return;
                var propertyName = "FirstName";
                Validate(value, propertyName);
                firstName = value;
                FirePropertyChanged(propertyName);
            }
        }

        private string lastName;
        [Required(ErrorMessage = "Required field")]
        [StringLength(40, ErrorMessage = "Cannot exceed 40")]
        [Display(Name = "Last Name", Description = "Employee's last name")]
        public string LastName
        {
            get { return lastName; }
            set
            {
                if (lastName == value) return;
                var propertyName = "LastName";
                Validate(value, propertyName);
                lastName = value;
                FirePropertyChanged(propertyName);
            }
        }

        private decimal annualSalary;
        [Required(ErrorMessage = "Salary is required")]
        [Range(0, 1000000)]
        [Display(Name="Salary $", Description = "Salary of the employee")]
        public decimal AnnualSalary
        {
            get { return annualSalary; }
            set
            {
                if (annualSalary == value) return;
                var propertyName = "AnnualSalary";
                Validate(value, propertyName);
                annualSalary = value;
                FirePropertyChanged(propertyName);
            }
        }

        private int age;
        [Required(ErrorMessage = "Age is required")]
        [Display(Name = "Age", Description = "Age of the employee")]
        //[Range(0, 150, ErrorMessage = "Value must be within range")]
        [CustomValidation(typeof(EmployeeValidator), "ValidateAge")]
        public int Age
        {
            get { return age; }
            set
            {
                if (age == value) return;
                var propertyName = "Age";
                Validate(value, propertyName);
                age = value;
                FirePropertyChanged(propertyName);
            }
        }

        private string email;
        [Display(Name = "Email", Description = "Email of the employee")]
        [RegularExpression(emailExpression, ErrorMessage = "Please enter a valid email address")]
        public string Email
        {
            get { return email; }
            set
            {
                if (email == value) return;
                var propertyName = "Email";
                Validate(value, propertyName);
                email = value;
                FirePropertyChanged(propertyName);
            }
        }

        private string phone;
        [Display(Name = "Phone", Description = "Employee's phone number (###) ###-####")]
        [RegularExpression(phoneExpression, ErrorMessage = "Phone number must match the format (###) ###-####")]
        public string Phone
        {
            get { return phone; }
            set
            {
                if (phone == value) return;
                var propertyName = "Phone";
                Validate(value, propertyName);
                phone = value;
                FirePropertyChanged(propertyName);
            }
        }

        #region Implementation of IEditableObject

        public override void BeginEdit()
        {
            this.cache = new Employee()
                             {
                                 EmployeeId = this.EmployeeId,
                                 Age = this.Age,
                                 AnnualSalary = this.AnnualSalary,
                                 Email = this.Email,
                                 FirstName = this.FirstName,
                                 LastName = this.LastName,
                                 Phone = this.Phone,
                             };
        }

        public override void CancelEdit()
        {
            this.EmployeeId = this.cache.EmployeeId;
            this.Age = this.cache.Age;
            this.AnnualSalary = this.cache.AnnualSalary;
            this.Email = this.cache.Email;
            this.FirstName = this.cache.FirstName;
            this.LastName = this.cache.LastName;
            this.Phone = this.cache.Phone;
        }

        public override void EndEdit()
        {
            this.cache = null;
            //    Validator.ValidateObject(this, new ValidationContext(this, null, null)); // true);
        }

        #endregion
    }
}

