using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;

namespace DataFormTest
{
    public abstract class ModelBase : INotifyPropertyChanged, IEditableObject
    {
        private List<ValidationResult> validationResults { get; set; }

        public event PropertyChangedEventHandler PropertyChanged;
        protected void FirePropertyChanged(string propertyname)
        {
            var handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyname));
        }

        protected void Validate(object value, string propertyName)
        {
            Validator.ValidateProperty(value, 
                new ValidationContext(this, null, null) { MemberName = propertyName });
        }

        public bool IsValid()
        {
            return Validator.TryValidateObject(this,
                new ValidationContext(this, null, null), this.validationResults, true);
        }

        #region Implementation of IEditableObject

        public abstract void BeginEdit();
        public abstract void EndEdit();
        public abstract void CancelEdit();

        #endregion
    }
}